from fastapi import APIRouter, Depends, HTTPException
from sqlmodel import Session, select
from backend.models.models import OrderItem
from backend.schemas.orderitem import OrderItemRead, OrderItemCreate, OrderItemUpdate, OrderItemList
from backend.database import get_session

router = APIRouter()

@router.post("/", response_model=OrderItemRead)
def create_order_item(order_item: OrderItemCreate, session: Session = Depends(get_session)):
    new_order_item = OrderItem(**order_item.dict())
    session.add(new_order_item)
    session.commit()
    session.refresh(new_order_item)
    return new_order_item

@router.get("/{order_item_id}", response_model=OrderItemRead)
def read_order_item(order_item_id: int, session: Session = Depends(get_session)):
    order_item = session.get(OrderItem, order_item_id)
    if not order_item:
        raise HTTPException(status_code=404, detail="Order item not found")
    return order_item

@router.get("/order/{order_id}", response_model=OrderItemList)
def list_order_items(order_id: int, session: Session = Depends(get_session)):
    items = session.exec(select(OrderItem).where(OrderItem.order_id == order_id)).all()
    return {"items": items}

@router.get("/{order_item_id}/subtotal", response_model=float)
def calculate_item_subtotal(order_item_id: int, session: Session = Depends(get_session)):
    order_item = session.get(OrderItem, order_item_id)
    if not order_item:
        raise HTTPException(status_code=404, detail="Order item not found")
    return order_item.price * order_item.quantity

@router.put("/{order_item_id}", response_model=OrderItemRead)
def update_order_item(order_item_id: int, order_item_update: OrderItemUpdate, session: Session = Depends(get_session)):
    order_item = session.get(OrderItem, order_item_id)
    if not order_item:
        raise HTTPException(status_code=404, detail="Order item not found")

    for key, value in order_item_update.dict(exclude_unset=True).items():
        setattr(order_item, key, value)

    session.add(order_item)
    session.commit()
    session.refresh(order_item)
    return order_item

@router.delete("/{order_item_id}")
def delete_order_item(order_item_id: int, session: Session = Depends(get_session)):
    order_item = session.get(OrderItem, order_item_id)
    if not order_item:
        raise HTTPException(status_code=404, detail="Order item not found")
    session.delete(order_item)
    session.commit()
    return {"message": "Order item deleted successfully"}
